**************************************************************************
** Program Name : GENDBC.PRG
** Creation Date: 94.12.01
**
** Purpose      : To take an existing FoxPro 3.0 database and
**                generate an output program that can be used
**                to "re-create" that database.
** Parameters   : cOutFile - A character string that contains
**                           the name of an output file.  This
**                           can contain path information and
**                           an extension.  If no extension is
**                           supplied, one will be provided.
**
** Modification History:
**
** 1994.12.01 KRT  Created program, runs on Build 329 of VFP
** 1994.12.02 KRT  Added GetView function and cleaned up all code
** 1994.12.05 KRT  Modified some areas and verified run on Build 335
** 1994.12.06 KRT  Added things for international conversion
** 1994.12.08 KRT  Added function ADBOBJECTS() to code
** 1994.12.12 KRT  Added commands COPY PROCEDURES TO
** 1995.01.04 KRT  Added connection properties
** 1995.01.05 KRT  Added support for long filenames - thierryp
** 1995.01.05 KRT  Added support for RI
** 1995.01.06 KRT  Added support for MS-DOS short filenames (NAME clause)
** 1995.01.08 KRT  Added status bar line
**************************************************************************
LPARAMETERS cOutFile

PRIVATE ALL EXCEPT g_*
*! Public Variables
IF SET("TALK") = "ON"                    && To restore SET TALK after use
	SET TALK OFF                         && -- Have to do it this way so
	g_cSetTalk = "ON"                    && -- nothing get's on screen
ELSE
	g_cSetTalk = "OFF"
ENDIF
g_cOnError = ON("ERROR")                 && To restore old ON ERROR condition
g_cSetDeleted = SET("DELETED")           && To restore SET DELETED later
g_cSetStatusBar = SET("STATUS BAR")      && To restore STATUS bar
g_cStatusText = SYS(2001, "MESSAGE", 1 ) && To restore text that may be on it
g_nMax = 7                               && For status line information
g_CurrentStat = 1                        && For status line information
SET DELETED ON
IF g_cSetStatusBar = "OFF"
    SET STATUS BAR ON
ENDIF

*! Our generic error handling routine
ON ERROR DO GenDBC_Error WITH MESSAGE( ), LINENO( )

#DEFINE CRLF CHR(13) + CHR(10)

**************************************************************************
** Error Messages
**************************************************************************
#DEFINE No_Database_In_Use "No Database is in use. " + ;
                           "This program will only work on the " + ;
                           "currently selected database."
   
#DEFINE Invalid_Parameters "Invalid Parameters..." + CRLF + ;
                           "An output file must be specified." + CRLF +;
                           'ie: DO GENDBC WITH "filename.prg"'
#DEFINE Invalid_Destination "Invalid Destination File "
#DEFINE No_Temp_File        "Could not create temporary file: "
#DEFINE No_Relation         "Could not find relation specified in Database"
#DEFINE Not_All_Relation    "Could not find all information for relation"
#DEFINE No_Parent           "Could not find parent table"
#DEFINE No_Output_Written   "Could not create or write to output file"
#DEFINE Error_Title         "Aborting GENDBC..."
#DEFINE Unrecoverable       "Unrecoverable Error: "
#DEFINE At_Line             " At Line: "
#DEFINE No_Find             "Could not set RI Information."

**************************************************************************
** Comments To Be Placed In Resulting Output File
**************************************************************************
#DEFINE Begin_Relation    "*************** Begin Relations Setup **************"
#DEFINE Begin_Table       "Table setup for "
#DEFINE Begin_Index       "Create each index for "
#DEFINE Begin_Prop        "Change properties (if any) for "
#DEFINE Begin_View        "View setup for "
#DEFINE Begin_Proc        "Procedure Re-Creation"
#DEFINE Begin_Connections "Connection Definitions"
#DEFINE Begin_RI          "Referential Integrity Setup"

#DEFINE Heading_1 "*       *********************************************************" + CRLF + ;
                  "*       *                                                        " + CRLF
#DEFINE Heading_2 "*       *                                                        " + CRLF + ;
                  "*       *********************************************************" + CRLF + ;
                  "*       *                                                        " + CRLF + ;
                  "*       * Description:                                           " + CRLF + ;
                  "*       * This program was automatically generated by GENDBC     " + CRLF + ;
                  "*       * Version 1.0                                            " + CRLF + ;
                  "*       *                                                        " + CRLF + ;
                  "*       *********************************************************" + CRLF

*! Check for valid parameters
IF PARAMETERS() < 1 OR TYPE( "cOutFile" ) <> "C"
    = FatalAlert( Invalid_Parameters, .F. )
ENDIF

*! Make sure a database is open
IF EMPTY( DBC( ))
    = FatalAlert( No_Database_In_Use, .F. )
ELSE
    g_cDatabase = DBC( )
    IF RAT( "\", g_cDatabase ) > 0
        g_cDatabase = SUBSTR( g_cDatabase, RAT( "\", g_cDatabase ) + 1 )
    ENDIF
    g_cFullDatabase = DBC( )
ENDIF

*! Check for proper extensions or add one if none specified
IF RAT( ".PRG", cOutFile ) = 0 AND RAT( ".", cOutFile ) = 0
    cOutFile = cOutFile + ".PRG"
ENDIF

*! Make sure the output file is valid
hFile = FCREATE( cOutFile )
IF hFile <= 0
    = FatalAlert( Invalid_Destination + cOutFile, .F. )
ENDIF

= FCLOSE( hFile )
ERASE ( cOutFile )

*! Remember all our tables that are open for this database
g_nTotal_Tables_Used = AUSED( g_aTables_Used )

*! Get number of tables contained in database
nTotal_Tables = ADBOBJECTS( aAll_Tables, "Table" )
g_nMax = g_nMax + nTotal_Tables
= Stat_Message( )

*! Get number of views contained in database
nTotal_Views = ADBOBJECTS( aAll_Views, "View" )
g_nMax = g_nMax + nTotal_Views
= Stat_Message( )

*! Get number of connections contained in database
nTotal_Connections = ADBOBJECTS( aAll_Connections, "Connection" )
g_nMax = g_nMax + nTotal_Connections
= Stat_Message( )

*! Get number of relations contained in database
nTotal_Relations = ADBOBJECTS( aAll_Relations, "Relation" )
g_nMax = g_nMax + nTotal_Relations
= Stat_Message( )
CLOSE DATABASE

SELECT 0
*! Check for this database... If it's there, we must have left it
*! here because of an error last time.
IF FILE( "GENDBC.DBF" )
    ERASE "GENDBC.DBF"
    ERASE "GENDBC.FPT"
ENDIF

CREATE TABLE GenDBC ( Program M )
APPEND BLANK
USE

**************************
*** Get Tables
**************************
*! Open the database
OPEN DATABASE ( g_cFullDatabase )
IF nTotal_Tables > 0
    FOR nLoop = 1 TO nTotal_Tables
        DO GetTable WITH ALLTRIM( aAll_Tables( nLoop )), "GenDBC.tmp"
        = Stat_Message( )
        USE GenDBC
        APPEND MEMO Program FROM "GenDBC.tmp"
        USE
        ERASE "GenDBC.tmp"
    ENDFOR
ENDIF

**************************
*** Get Connections
**************************
IF nTotal_Connections > 0
    FOR nLoop = 1 TO nTotal_Connections
        DO GetConn WITH aAll_Connections( nLoop ), "GenDBC.tmp"
        = Stat_Message( )
        USE GenDBC
        APPEND MEMO Program FROM "GenDBC.tmp"
        USE
        ERASE "GenDBC.tmp"
    ENDFOR
ENDIF

**************************
*** Get Views
**************************
*! Ensure that there are Views to parse
IF nTotal_Views > 0
    FOR nLoop = 1 TO nTotal_Views
        DO GetView WITH ALLTRIM( aAll_Views( nLoop )), "GenDBC.tmp"
        = Stat_Message( )
        USE GenDBC
        APPEND MEMO Program FROM "GenDBC.tmp"
        USE
        ERASE "GenDBC.tmp"
    ENDFOR
ENDIF

**************************
*** Get Relations
**************************
IF nTotal_Relations > 0
    USE GenDBC
    REPLACE Program WITH Begin_Relation + CRLF ADDITIVE
    FOR nLoop = 1 TO nTotal_Relations
        REPLACE Program WITH "ALTER TABLE '" + aAll_Relations( nLoop, 1 ) +;
                             "' ADD FOREIGN KEY TAG " +;
                             aAll_Relations( nLoop, 3 ) +;
                             " REFERENCES " + ;
                             aAll_Relations( nLoop, 2 ) +;
                             " TAG " + aAll_Relations( nLoop, 4 ) +;
                             CRLF ADDITIVE
    = Stat_Message( )
    ENDFOR
ENDIF

**************************
*** Get Stored Procedures
**************************
COPY PROCEDURES TO "GenDBC.tmp"
= ADIR( aTemp, "GenDBC.tmp" )

*! Check for actual output file being created
IF aTemp( 1, 2 ) > 0
    **************************
    *** Make the output file
    *** re-create the proced-
    *** ure file via code.
    **************************
    hFile = FOPEN( "GenDBC.tmp" )
    IF hFile <= 0
        = FatalAlert( No_Output_Written, .T. )
    ENDIF

    hOutFile = FCREATE( "GenDBC.$$$" )
    IF hOutFile <= 0
        = FCLOSE( hFile )
        = FatalAlert( No_Output_Written, .T. )
    ENDIF
    = WriteFile( hOutFile, "" )
    = WriteFile( hOutFile, "********* " + Begin_Proc + " *********")
    = WriteFile( hOutFile, "hFile = FCREATE( 'GENDBC.TMP' )" )
    = WriteFile( hOutFile, "IF hFile <= 0" )
    = WriteFile( hOutFile, "     RETURN" )
    = WriteFile( hOutFile, "ENDIF" )
    = WriteFile( hOutFile, "" )
    = WriteFile( hOutFile, "cOldConsole = SET('CONSOLE')" )
    = WriteFile( hOutFile, "SET CONSOLE OFF" )
    = WriteFile( hOutFile, "nOldText = _TEXT" )
    = WriteFile( hOutFile, "_TEXT = hFile" )
    = WriteFile( hOutFile, "TEXT" )
    DO WHILE !FEOF( hFile )
        = WriteFile( hOutFile, SPACE(4) + FGETS( hFile ))
    ENDDO
    = WriteFile( hOutFile, "ENDTEXT" )
    = WriteFile( hOutFile, "_TEXT = nOldText" )
    = WriteFile( hOutFile, "= FCLOSE( hFile )" )
    = WriteFile( hOutFile, "APPEND PROCEDURES FROM 'GENDBC.TMP'" )
    = WriteFile( hOutFile, "ERASE 'GENDBC.TMP'" )
    = WriteFile( hOutFile, "IF cOldConsole = 'ON'" )
    = WriteFile( hOutFile, "    SET CONSOLE ON" )
    = WriteFile( hOutFile, "ENDIF" )
    = WriteFile( hOutFile, "" )
    = FCLOSE( hFile )
    = FCLOSE( hOutFile )
    USE GenDBC
    APPEND MEMO Program FROM "GENDBC.$$$"
    ERASE "GENDBC.$$$"
    USE
ENDIF
ERASE "GENDBC.TMP"
= Stat_Message( )

CLOSE DATABASE  && Because we're going to start peeking into the
                && table structure of the DBC

**************************
*** Get RI Info
**************************
IF nTotal_Relations > 0
    DO GetRI WITH "GenDBC.tmp"
    IF FILE( "GenDBC.tmp" )
        USE GenDBC
        APPEND MEMO Program FROM "GenDBC.tmp"
        USE
        ERASE "GenDBC.tmp"
    ENDIF
ENDIF
= Stat_Message( )

*! Make it a permanent file
USE GenDBC
REPLACE Program WITH Heading_1 + "*       * " + DTOC( DATE( )) +;
                     SPACE( 19 - LEN( g_cDatabase ) / 2 ) + ;
                     g_cDatabase + SPACE( 19 - LEN( g_cDatabase ) / 2 ) +;
                     TIME() + CRLF + Heading_2 + CRLF +;
                     "CREATE DATABASE '" + g_cDatabase + "'" + ;
                    CRLF + CRLF + Program
COPY MEMO Program TO ( cOutFile )
USE
ERASE "GenDBC.DBF"
ERASE "GenDBC.FPT"
= Stat_Message( )

*! Exit Program
= GenDBC_CleanUp()
*********************** END OF PROGRAM ***********************

**************************************************************************
**
** Function Name: GETRI( <ExpC> )
** Creation Date: 1994.12.02
** Purpose      :
**
**      To take existing FoxPro 3.0 RI Infomration, and generate an output
**      program that can be used to "re-create" this.
**
** Parameters   :
**
**      cOutFileName - A character string containing the name of the
**                     output file
**
** Modification History:
**
**  1995.01.05  KRT     Created function
**************************************************************************
PROCEDURE GetRI
    LPARAMETERS cOutFileName

    PRIVATE ALL EXCEPT g_*

    *! Create the output file
    hGTFile = FCREATE( cOutFileName )
    IF hGTFile < 1
        = FatalAlert( No_Temp_File + cOutFileName, .T. )
    ENDIF

    *! USE the database
    USE ( g_cFullDatabase )

    LOCATE FOR ObjectType = "Relation" AND !EMPTY( RiInfo )
    IF FOUND()
        = WriteFile( hGTFile, "***** " + Begin_RI + " *****" )
        = WriteFile( hGTFile, "CLOSE DATABASE" )
        = WriteFile( hGTFile, "USE '" +  g_cDatabase + "'" )
        DO WHILE FOUND()
            nStart = 1
            cITag = ""
            cTable = ""
            cRTag = ""
            DO WHILE nStart <= LEN( Property )
                nSize = ASC( SUBSTR( Property, nStart, 1 )) +;
                ( ASC( SUBSTR( Property, nStart + 1, 1 )) * 256 ) +;
                ( ASC( SUBSTR( Property, nStart + 2, 1 )) * 256^2 ) + ;
                ( ASC( SUBSTR( Property, nStart + 3, 1 )) * 256^3 )

                nKey = ASC( SUBSTR( Property, nStart + 6, 1 ))

                DO CASE
                    CASE nKey = 13
                        cITag = SUBSTR( Property, nStart + 7, nSize - 8 )
                    CASE nKey = 18
                        cTable = SUBSTR( Property, nStart + 7, nSize - 8 )
                    CASE nKey = 19
                        cRTag = SUBSTR( Property, nStart + 7, nSize - 8 )
                ENDCASE
                nStart = nStart + nSize
            ENDDO
            = WriteFile( hGTFile, "LOCATE FOR ObjectType = 'Relation' AND '" + cITag + "'$Property AND '" + cTable + "'$Property AND '" + cRTag + "'$Property" )
            = WriteFile( hGTFile, "IF FOUND()" )
            = WriteFile( hGTFile, "     REPLACE RiInfo WITH '" + RiInfo + "'" )
            = WriteFile( hGTFile, "ELSE" )
            = WriteFile( hGTFile, "     ? '" + No_Find + "'" )
            = WriteFile( hGTFile, "ENDIF" )
            CONTINUE
        ENDDO
        = FCLOSE( hGTFile )
    ELSE
        = FCLOSE( hGTFile )
    ERASE ( cOutFileName )
    ENDIF
    USE
RETURN

**************************************************************************
**
** Function Name: GETTABLE( <ExpC>, <ExpC> )
** Creation Date: 1994.12.01
** Purpose      :
**
**        To take an existing FoxPro 3.0 View, and generate an output
**        program that can be used to "re-create" that view.
**
** Parameters   :
**
**        cTableName   - A character string representing the name of the
**                       existing view
**        cOutFileName - A character string containing the name of the
**                       output file
**
** Modification History:
**
**  1994.12.02  KRT     Created function
**  1994.12.05  KRT     Made it a function and cleaned it up
**  1994.12.08  KRT     Assume Database is open to speed up operation
**************************************************************************
PROCEDURE GetTable
    LPARAMETERS cTableName, cOutFileName

    PRIVATE ALL EXCEPT g_*

    *! Create the output file
    hGTFile = FCREATE( cOutFileName )
    IF hGTFile < 1
        = FatalAlert( No_Temp_File + cOutFileName, .T. )
    ENDIF

    *! Open Table to get field info
    USE ( cTableName ) && Open it up to get field info

    *! Get all the fields
    nNumberOfFields = AFIELDS( aAll_Fields )

    *! Header Information
    = WriteFile( hGTFile, "***** " + Begin_Table + cTableName + " *****" )

    *! If the table is greater than 8 characters, we use the "NAME" clause
    IF LEN( cTableName ) > 8
        cCreateTable = "CREATE TABLE '" + SUBSTR( cTableName, 1, 8 ) + "' NAME '" + cTableName + "' ("
    ELSE
        cCreateTable = "CREATE TABLE '" + cTableName + "' ("
    ENDIF
    
    *! Information about each field that can been written with CREATE TABLE - SQL
    FOR nInner_Loop = 1 TO nNumberOfFields
        IF nInner_Loop = 1
            cCreateTable = cCreateTable + aAll_Fields( nInner_Loop, 1 ) + " "
        ELSE
            cCreateTable = SPACE( LEN( cTableName ) + 15 ) + ;
                           aAll_Fields( nInner_Loop, 1 ) + " "
        ENDIF
        cCreateTable = cCreateTable + aAll_Fields( nInner_Loop, 2 )
        DO CASE
            CASE aAll_Fields( nInner_Loop, 2 ) == "C"
                cCreateTable = cCreateTable + "(" + ;
                           ALLTRIM( STR( aAll_Fields( nInner_Loop, 3 ))) + ")"
            CASE aAll_Fields( nInner_Loop, 2 ) == "N" OR ;
                aAll_Fields( nInner_Loop, 2 ) == "F"
                cCreateTable = cCreateTable + "(" + ;
                ALLTRIM( STR( aAll_Fields( nInner_Loop, 3 ))) + ;
                ", " + ALLTRIM( STR( aAll_Fields( nInner_Loop, 4 ))) + ")"
            CASE aAll_Fields( nInner_Loop, 2 ) == "B"
                cCreateTable = cCreateTable + "(" + ;
                ALLTRIM( STR( aAll_Fields( nInner_Loop, 4 ))) ;
                + ")"
        ENDCASE

        IF aAll_Fields( nInner_Loop, 5 )
            cCreateTable = cCreateTable + " NULL"
        ELSE
            cCreateTable = cCreateTable + " NOT NULL"
        ENDIF

        *! Get properties for fields
        cFieldAlias = cTableName + "." + aAll_Fields( nInner_Loop, 1)

        cTemp = DBGETPROP( cFieldAlias, "Field", "RuleExpression" )
        IF !EMPTY( cTemp )
            cCreateTable = cCreateTable + " CHECK " + cTemp
        ENDIF

        cTemp = DBGETPROP( cFieldAlias, "Field", "RuleText" )
        IF !EMPTY( cTemp )
            cCreateTable = cCreateTable + " ERROR " + cTemp
        ENDIF

        cTemp = DBGETPROP( cFieldAlias, "Field", "DefaultValue" )
        IF !EMPTY( cTemp )
            cCreateTable = cCreateTable + " DEFAULT " + cTemp
        ENDIF

        IF nInner_Loop <> nNumberOfFields
            cCreateTable = cCreateTable + ", ;"
        ELSE
            cCreateTable = cCreateTable + ")"
        ENDIF

        = WriteFile( hGTFile, cCreateTable )
    ENDFOR

    *! Get Index Information
    = WriteFile( hGTFile, CRLF + "***** " + Begin_Index + cTableName + " *****" )
    FOR nInner_Loop = 1 TO TAGCOUNT()
        cTag = UPPER( ALLTRIM( TAG( nInner_Loop )))
        IF !EMPTY( cTag )
            IF PRIMARY( nInner_Loop )
                = WriteFile( hGTFile, "ALTER TABLE '" + cTableName + ;
                                      "' ADD PRIMARY KEY " + SYS( 14, nInner_Loop ) ;
                                      + " TAG " + cTag )
            ELSE
                = WriteFile( hGTFile, "INDEX ON " + SYS(14, nInner_Loop ) ;
                                      + " TAG " + cTag )
            ENDIF
        ELSE
            EXIT FOR
        ENDIF
    ENDFOR

    *! Get Properties For Table
    = WriteFile( hGTFile, CRLF + "***** " + Begin_Prop + cTableName + " *****" )
    FOR nInner_Loop = 1 TO nNumberOfFields
        cFieldAlias = cTableName + "." + aAll_Fields( nInner_Loop, 1)
        cTemp = DBGETPROP( cFieldAlias, "Field", "Caption" )
        IF !EMPTY( cTemp )
            cTemp = STRTRAN( cTemp, ["], ['] )
            = WriteFile( hGTFile, [= DBSETPROP( "] + cFieldAlias + ;
                                  [", "Field", "Caption", "] + cTemp + ;
                                  [" )] )
        ENDIF
        cTemp = DBGETPROP( cFieldAlias, "Field", "Comment" )
        IF !EMPTY( cTemp )
            cTemp = STRTRAN( cTemp, ["], ['] )
            = WriteFile( hGTFile, [= DBSETPROP( "] + cFieldAlias + ;
                                  [", "Field", "Comment", "] + cTemp ;
                                  + [" )] )
        ENDIF
    ENDFOR
    cTemp = DBGETPROP( cTableName, "Table", "Comment" )
    IF !EMPTY( cTemp )
        cTemp = STRTRAN( cTemp, ["], ['] )
        = WriteFile( hGTFile, [= DBSETPROP( "] + cTableName + ;
                              [", "Table", "Comment", "] + cTemp + [")] )
    ENDIF

    cTemp = DBGETPROP( cTableName, "Table", "DeleteTrigger" )
    IF !EMPTY( cTemp )
        = WriteFile( hGTFile, "CREATE TRIGGER ON '" + cTableName + ;
                              "' FOR DELETE AS " + cTemp )
    ENDIF

    cTemp = DBGETPROP( cTableName, "Table", "InsertTrigger" )
    IF !EMPTY( cTemp )
        = WriteFile( hGTFile, "CREATE TRIGGER ON '" + cTableName + ;
                              "' FOR INSERT AS " + cTemp )
    ENDIF

    cTemp = DBGETPROP( cTableName, "Table", "UpdateTrigger" )
    IF !EMPTY( cTemp )
        = WriteFile( hGTFile, "CREATE TRIGGER ON '" + cTableName + ;
                              "' FOR UPDATE AS " + cTemp )
    ENDIF

    cTemp = DBGETPROP( cTableName, "Table", "RuleExpression" )
    IF !EMPTY( cTemp )
        cError = DBGETPROP( cTableName, "Table", "RuleText" )
        IF !EMPTY( cError )
            = WriteFile( hGTFile, "ALTER TABLE '" + cTableName + ;
                                  "' SET CHECK " + cTemp + " ERROR " + ;
                                  cError )
        ELSE
            = WriteFile( hGTFile, "ALTER TABLE '" + cTableName + ;
                                  "' SET CHECK " + cTemp )
        ENDIF
    ENDIF
    = WriteFile( hGTFile, "" )
    = FCLOSE( hGTFile )
RETURN

**************************************************************************
**
** Function Name: GETVIEW( <ExpC>, <ExpC> )
** Creation Date: 1994.12.01
** Purpose      :
**
**        To take an existing FoxPro 3.0 View, and generate an output
**        program that can be used to "re-create" that view.
**
** Parameters   :
**
**        cViewName    - A character string representing the name of the 
**                       existing view
**        cOutFileName - A character string containing the name of the 
**                       output file
**
** Modification History:
**
**  1994.12.01  JHL   Created Program, runs on Build 329 of FoxPro 3.0
**  1994.12.02  KRT   Added to GenDBC, removed third parameter, cleaned up
**  1994.12.08  KRT   Assume Database is open to speed up operation
***************************************************************************************
PROCEDURE GetView

    LPARAMETERS cViewName, cOutFileName

    PRIVATE ALL EXCEPT g_*

    nFileHand = FCREATE( cOutFileName, 0 )
    IF nFileHand < 1
        = FatalAlert( No_Temp_File + cOutFileName, .T. )
    ENDIF

    *! Get View Information for later use
    nSourceType = DBGetProp( cViewName, 'View', 'SourceType' )
    cConnectName = ALLTRIM( DBGetProp( cViewName, 'View', 'ConnectName' ))
    cSQL = ALLTRIM( DBGetProp( cViewName, 'View', 'SQL' ))

    cnUpdateType = ALLTRIM( STR( DBGetProp( cViewName, 'View', 'UpdateType' )))
    cnWhereType = ALLTRIM( STR( DBGetProp( cViewName, 'View', 'WhereType' )))
    clFetchMemo = IIF( DBGetProp( cViewName, 'View', 'Fetchmemo' ),'.T.','.F.' )
    clShareConnection = IIF( DBGetProp( cViewName, 'View', 'ShareConnection' ),'.T.','.F.' )
    clSendUpdates = IIF( DBGetProp( cViewName, 'View', 'SendUpdates' ),'.T.','.F.' )
    cnUseMemoSize = ALLTRIM( STR( DBGetProp( cViewName, 'View', 'UseMemoSize' )))
    cnFetchSize = ALLTRIM( STR( DBGetProp( cViewName, 'View', 'FetchSize' )))
    cnMaxRecords = ALLTRIM( STR( DBGetProp( cViewName, 'View', 'MaxRecords' )))
    ccTables = ALLTRIM( DBGetProp( cViewName, 'View', 'Tables' ))

    *! Generate Comment Block
    cCommentBlock = "***************** " + Begin_View + cViewName + ;
        " ***************" + CRLF

    = WriteFile( nFileHand, cCommentBlock )

    *! Generate CREATE VIEW command
    cCreateString = 'CREATE SQL VIEW "'+ALLTRIM( cViewName )+'" ; '+CRLF

    IF nSourceType != 1    && If it isn't a local view
        cCreateString = cCreateString + '   REMOTE '
        IF !EMPTY( cConnectName )
            cCreateString = cCreateString + 'CONNECT "' + cConnectName + '" ; '+CRLF
        ENDIF
    ENDIF
    cCreateString = cCreateString + '   AS '+ cSQL + CRLF

    = WriteFile( nFileHand, cCreateString )

    *! GENERATE code to Set View Level Properties
    cViewDBSetPrefix = [=DBSetProp( ']+cViewName+[', 'View', ]

    cViewProps = '****' + CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['UpdateType', ] + cnUpdateType + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['WhereType', ] + cnWhereType + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['FetchMemo', ] + clFetchMemo + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['SendUpdates', ] + clSendUpdates + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['UseMemoSize', ] + cnUseMemoSize + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['FetchSize', ] + cnFetchSize + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['MaxRecords', ] + cnMaxRecords + [ )]+ CRLF
    cViewProps = cViewProps + cViewDBSetPrefix + ;
     ['Tables', '] + ccTables + [' )]+ CRLF

    IF nSourceType != 1    && If it isn't a local view
        cViewProps = cViewProps + cViewDBSetPrefix + ;
                     ['ShareConnection', ] + clShareConnection + [ )]+ CRLF
    ENDIF

    = WriteFile( nFileHand, cViewProps )

    *! GENERATE code to Set Field Level Properties
    USE ( DBC(  )) AGAIN IN 0 ALIAS GenViewCursor
    SELECT GenViewCursor
    LOCATE FOR UPPER( GenViewCursor.ObjectName ) = cViewName AND ;
                      GenViewCursor.ObjectType = 'View'
    nObjectId = GenViewCursor.ObjectId
    SELECT ObjectName FROM GenViewCursor ;
           WHERE GenViewCursor.ParentId = nObjectId ;
           INTO ARRAY aViewFields
    USE in GenViewCursor

    cFieldProps = '*! Field Level Properties for ' + cViewName + CRLF
    FOR nLoop = 1 TO ALEN( aViewFields, 1 )
        cFieldName = ALLTRIM( aViewFields( nLoop, 1 ))
        clKeyField = IIF( DBGetProp( cViewName+'.'+cFieldName, 'Field', 'KeyField' ),'.T.','.F.' )
        clUpdatable = IIF( DBGetProp( cViewName+'.'+cFieldName, 'Field', 'Updatable' ),'.T.','.F.' )
        ccUpdateName = ALLTRIM( DBGetProp( cViewName+'.'+cFieldName, 'Field', 'UpdateName' ))
        cViewFieldSetPrefix = [=DBSetProp( ']+cViewName+[.]+cFieldName+[', 'Field', ]
        cFieldProps = cFieldProps + '* Props for the '+cViewName+[.]+cFieldName+' field.' + CRLF
        cFieldProps = cFieldProps + cViewFieldSetPrefix + ['KeyField', ] + clKeyField + [ )]+ CRLF
        cFieldProps = cFieldProps + cViewFieldSetPrefix + ['Updatable', ] + clUpdatable + [ )]+ CRLF
        cFieldProps = cFieldProps + cViewFieldSetPrefix + ['UpdateName', '] + ccUpdateName + [' )]+ CRLF
        cFieldProps = cFieldProps + CRLF
    ENDFOR

    = WriteFile( nFileHand, cFieldProps )

    *! Close output file
    =FCLOSE( nFileHand )

RETURN

**************************************************************************
**
** Function Name: GETCONN( <ExpC>, <ExpC> )
** Creation Date: 1994.01.03
** Purpose      :
**
**        To take an existing FoxPro 3.0 Connection, and generate an output
**        program that can be used to "re-create" that connection.
**
** Parameters   :
**
**        cConnectName - A character string representing the name of the 
**                       existing connection
**        cOutFileName - A character string containing the name of the 
**                       output file
**
** Modification History:
**
**  1994.01.03  JHL   Created Program, runs on Build 329 of FoxPro 3.0
***************************************************************************************
PROCEDURE GetConn
    LPARAMETERS cConnectionName, cOutFileName

    PRIVATE ALL EXCEPT g_*

    nFileHand = FCREATE( cOutFileName, 0 )
    IF nFileHand < 1
        = FatalAlert( No_Temp_File + cOutFileName, .T. )
    ENDIF

    *! Get Connection Information for later use
    clAsynchronous = IIF( DBGetProp( cConnectionName, 'Connection', 'Asynchronous' ),'.T.','.F.' )
    clBatchMode = IIF( DBGetProp( cConnectionName, 'Connection', 'BatchMode' ),'.T.','.F.' )
    ccComment = ALLTRIM( DBGetProp( cConnectionName, 'Connection', 'Comment' ))
    ccConnectString = ALLTRIM( DBGetProp( cConnectionName, 'Connection', 'ConnectString' ))
    cnConnectTimeOut = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'ConnectTimeOut' )))
    ccDataSource = ALLTRIM( DBGetProp( cConnectionName, 'Connection', 'DataSource' ))
    cnDispLogin = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'DispLogin' )))
    clDispWarnings = IIF( DBGetProp( cConnectionName, 'Connection', 'DispWarnings' ),'.T.','.F.' )
    cnIdleTimeOut = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'IdleTimeOut' )))
    ccPassword = ALLTRIM( DBGetProp( cConnectionName, 'Connection', 'Password' ))
    cnQueryTimeOut = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'QueryTimeOut' )))
    cnTransactions = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'Transactions' )))
    ccUserId = ALLTRIM( DBGetProp( cConnectionName, 'Connection', 'UserId' ))
    cnWaitTime = ALLTRIM( STR( DBGetProp( cConnectionName, 'Connection', 'WaitTime' )))

    *! Generate Comment Block
    cCommentBlock = "***************** " + Begin_Connections + " " + cConnectionName + ;
        " ***************" + CRLF

    = WriteFile( nFileHand, cCommentBlock )

    *! Generate CREATE Connection command
    cCreateString = 'CREATE CONNECTION '+ALLTRIM( cConnectionName )+' ; '+CRLF

    IF EMPTY( ALLTRIM( ccConnectString ))  && If connectstring not specified
        cCreateString = cCreateString + '   DATASOURCE "' + ALLT( ccDataSource ) + '" ; ' + CRLF
        cCreateString = cCreateString + '   USERID "' + ALLT(ccUserId) + '" ; ' + CRLF
        cCreateString = cCreateString + '   PASSWORD "'+ ALLT( ccPassword ) + '"' + CRLF
    ELSE
        cCreateString = cCreateString + '   CONNSTRING "' + ALLT( ccConnectString ) + '"'
    ENDIF

    = WriteFile( nFileHand, cCreateString )

    *! GENERATE code to Set Connection Level Properties
    cConnectionDBSetPrefix = [=DBSetProp( ']+cConnectionName+[', 'Connection', ]

    cConnectionProps = '****' + CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['Asynchronous', ] + clAsynchronous + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['BatchMode', ] + clBatchMode + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['Comment', '] + ccComment + [' )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['DispLogin', ] + cnDispLogin + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['ConnectTimeOut', ] + cnConnectTimeOut + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['DispWarnings', ] + clDispWarnings + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['IdleTimeOut', ] + cnIdleTimeOut + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['QueryTimeOut', ] + cnQueryTimeOut + [ )]+ CRLF
    cConnectionProps = cConnectionProps + cConnectionDBSetPrefix + ;
                       ['Transactions', ] + cnTransactions + [ )]+ CRLF
    = WriteFile( nFileHand, cConnectionProps )
    *! Close output file
    =FCLOSE( nFileHand )
RETURN

**************************************************************************
**
** Function Name: FATALALERT( <ExpC> )
** Creation Date: 1994.12.02
** Purpose      :
**
**        Place a message box to alert user of a fatal error.
**
** Parameters   :
**
**        cAlert_Message - Message to display to user
**        lCleanup       - If we should try to restore environment
**
** Modification History:
**
**    1994.12.02  KRT        Added to GenDBC
**************************************************************************
PROCEDURE FatalAlert
    LPARAMETERS cAlert_Message, lCleanup

    = MESSAGEBOX( cAlert_Message, 16, Error_Title )

    IF lCleanup
        = GenDBC_CleanUp()
    ENDIF

    CANCEL
RETURN

**************************************************************************
*** Restore System Environment
**************************************************************************
PROCEDURE GenDBC_CleanUp
    *! Restore everything
    ON ERROR &g_cOnError
    SET TALK &g_cSetTalk
    SET TALK &g_cSetDeleted
    IF g_cSetStatusBar = "OFF"
        SET STATUS BAR OFF
	ENDIF
    SET MESSAGE TO ( g_cStatusText )
    CLOSE ALL

    OPEN DATABASE (g_cFullDatabase)
    IF g_nTotal_Tables_Used > 0
        FOR nLoop = 1 TO ALEN( g_aTables_Used ) / 2
            USE ( g_aTables_Used( nLoop, 1 )) IN ( g_aTables_Used( nLoop, 2 ))
        ENDFOR
    ENDIF
RETURN

**************************************************************************
**
** Function Name: WRITEFILE( <ExpN>, <ExpC> )
** Creation Date: 1994.12.02
** Purpose      :
**
**        Centralized file output routine to check for proper output
**
** Parameters   :
**
**        hFileHandle  - Handle of output file
**        cText        - Contents to write to file
**
** Modification History:
**
**    1994.12.02  KRT        Added to GenDBC
**************************************************************************
PROCEDURE WriteFile
    LPARAMETERS hFileHandle, cText

    nBytesSent = FPUTS( hFileHandle, cText )
    IF  nBytesSent < LEN( cText )
        = FatalAlert( No_Output_Written, .T. )
    ENDIF
RETURN

**************************************************************************
**
** Function Name: GenDBC_Error( <expC>, <expN> )
** Creation Date: 1994.12.02
** Purpose      :
**
**        Generalized Error Routine
**
** Parameters   :
**
**        cMess     - Message to give user
**        nLineNo   - Line Number Error Occurred
**
** Modification History:
**
**    1994.12.02  KRT        Added to GenDBC
**************************************************************************
PROCEDURE GenDBC_Error
    LPARAMETERS cMess, nLineNo

    = FatalAlert( Unrecoverable + CRLF + cMess + CRLF + ;
                  At_Line + ALLTRIM( STR( nLineNo )), .T. )
RETURN

**************************************************************************
**
** Function Name: Stat_Message( )
** Creation Date: 1994.01.08
** Purpose      :
**
**        Generalized Status Bar Progression
**
** Parameters   :
**
**        None
**
** Modification History:
**
**    1994.01.08  KRT        Added to GenDBC
**************************************************************************
PROCEDURE Stat_Message
    PRIVATE ALL EXCEPT g_*
    
    nStat = g_CurrentStat * ( 160 / g_nMax )
    SET MESSAGE TO REPLICATE( "|", nStat ) + " " + ALLTRIM( STR( INT( 100 * ( g_CurrentStat / g_nMax )))) + "%"
    g_CurrentStat = g_CurrentStat + 1
RETURN
